"""Load archiver configuration data
"""
from .errors import DefaultArchiverNotFound
from pkg_resources import resource_filename, Requirement
import configparser
import logging
from abc import ABCMeta, abstractproperty

logger = logging.getLogger('bact-archiver')


class ArchiverConfigurationInterface(metaclass=ABCMeta):

    @abstractproperty
    def retrieval_url(self):
        raise NotImplementedError('Derive from this class')

    @abstractproperty
    def management_url(self):
        raise NotImplementedError('Derive from this class')


class ArchiverConfiguration(ArchiverConfigurationInterface):
    """
    Args:
        name: short name or nickname of the archiver
        base_url : the base url of the archiver

    Retrieval path can be url.
    If base_url is not given, it is assumed that the retrieval path
    and management path contain the server path too.
    management_port can be only used if a base_url is given.
    """
    def __init__(self, name=None, base_url=None, retrieval_path='retrieval',
                 management_path='mgmt', management_port=17665,
                 description=""):
        """
        """
        self.name = name
        self.base_url = base_url
        self.retrieval_path = retrieval_path
        self.management_path = management_path
        self.management_port = management_port
        self.description = description

    def __repr__(self):
        args_text = "base_url={}, ".format(self.base_url)
        args_text += "retrieval_path={}, ".format(self.retrieval_path)
        args_text += "management_port={}, ".format(self.management_port)
        args_text += "management_path={}, ".format(self.management_path)
        txt = "{}({}, {})".format(self.__class__.__name__, self.name,
                                  args_text)
        return txt

    @property
    def retrieval_url(self):
        if self.base_url is not None:
            path = self.base_url + '/' + self.retrieval_path
            return path
        assert(self.retrieval_path is not None)
        return self.retrieval_path

    @property
    def management_url(self):
        if self.base_url is None:
            if self.management_port is not None:
                txt = (
                    "Management port {} but no base url is given  for archiver"
                    " {}. Do not know how to handle that!"
                ).format(self.management_path, self.name)
                raise AssertionError(txt)
            assert(self.management_path is not None)
            return self.management_path

        tmp = self.base_url, self.management_port, self.management_path
        path = "{}:{}/{}".format(tmp)
        return path


def get_config_filename(module_name):
    '''Get the configuration filename using pkg_resources
    '''
    requirement = Requirement.parse(module_name)
    filename = resource_filename(requirement, 'config/archiver.cfg')
    logger.info('Found config file %s', filename)
    return filename


def archiver_configurations(module_name):
    '''Generates :class:`ArchiverConfiguration` from configuration file

    Args:
        module_name : the name of the module containing the archiver
                      information

    Returns:
        dictonary of archiver configurations

    Retrieves the filename in question using :func:`get_config_filename`
    The key 'default' contains the same instance, that will be given by its
    orginal name
    '''
    filename = get_config_filename(module_name=module_name)
    config = configparser.ConfigParser()
    config.read(filename)

    def create_archiver(name):
        if name == "default":
            return
        kwargs = config[name]
        return ArchiverConfiguration(name=name, **kwargs)

    sections = config.sections()
    archiver_configs = {key: create_archiver(key) for key in sections}

    # Add default archiver as key default
    try:
        default_section = config['default']
    except Exception:
        txt = 'Could not find section default in file {}'.format(filename)
        txt = 'Known sections {}'.format(sections)
        logger.error(txt)
        raise

    name = default_section['name']
    try:
        default_archiver_cfg = archiver_configs[name]
    except KeyError as ke:
        txt = 'Could not find a section with name {}'.format(name)
        txt += ' known sections are {}'.format(sections)
        txt += ' archiver dic has keys {}'.format(
            list(archiver_configs.keys())
        )
        txt += ' Original exception {}'.format(ke)
        logger.error(txt)
        raise DefaultArchiverNotFound(txt)

    archiver_configs['default'] = default_archiver_cfg
    return archiver_configs



from . import epics_event_pb2 as proto

#: select protocol buffer decoder for EPICS type
decoder = {
    0: proto.ScalarString,
    1: proto.ScalarShort,
    2: proto.ScalarFloat,
    3: proto.ScalarEnum,
    4: proto.ScalarByte,
    5: proto.ScalarInt,
    6: proto.ScalarDouble,
    7: proto.VectorString,
    8: proto.VectorShort,
    9: proto.VectorFloat,
    10: proto.VectorEnum,
    11: proto.VectorChar,
    12: proto.VectorInt,
    13: proto.VectorDouble,
    14: proto.V4GenericBytes
}

#: translate EPICS types to numpy types
dtypes = {
    0: 'S40',
    1: 'i2',
    2: 'f4',
    3: 'u4',
    4: 'b1',
    5: 'i4',
    6: 'f8',
    7: 'str',
    8: 'i2',
    9: 'f4',
    10: 'u4',
    11: 'b1',
    12: 'i4',
    13: 'f8'
}

dsize = {
    0: 40,
    1: 2,
    2: 4,
    3: 4,
    4: 1,
    5: 4,
    6: 8,
    7: 40,
    8: 2,
    9: 4,
    10: 4,
    11: 1,
    12: 4,
    13: 8
}

dbrtypes = {
    'STRING': 0,
    'INT': 1,
    'SHORT': 1,
    'FLOAT': 2,
    'ENUM': 3,
    'CHAR': 4,
    'LONG': 5,
    'DOUBLE': 6
}


class Chunk(object):
    def __init__(self, header):
        self.header = header
        self.events = []
        self.meta = None
        self.value = None

    def add(self, event):
        self.events.append(event)



# -*- coding: utf-8 -*-
# Author: Andreas Schälicke <andreas.schaelicke@helmholtz-berlin.de>
# Time-stamp: <2017-11-21 11:49:27 schaelicke>
"""Work horse access to EPICS archiver appliance
"""

from .epics_event import read_chunk, decode
from . import epics_event_pb2 as proto
from .protocol_buffer import (Chunk, dtypes as _dtypes, decoder as _decoder,
                              dbrtypes as _dbrtypes, dsize as _dsize)
from .archiver import ArchiverBasis, convert_datetime_to_timestamp

from urllib.request import urlopen, quote, HTTPError
from functools import lru_cache

import numpy as np
import pandas as pd

import datetime
import dateutil.parser
import dateutil.tz
import enum

import logging

logger = logging.getLogger('bact-archiver')


class ApplianceOperators(enum.Enum):
    '''

    Todo:
       check: generic archiver request?
    '''
    firstSample =   'firstSample'
    lastSample =    'lastSample'
    firstFill =     'firstFill'
    lastFill =      'lastFill'
    mean =          'mean'
    min =           'min'
    max =           'max'
    count =         'count'
    ncount =        'ncount'
    nth =           'nth'
    median =        'median'
    std =           'std'
    jitter =        'jitter'
    ignoreflyers =  'ignoreflyers'
    flyers =        'flyers'
    variance =      'variance'
    popvariance =   'popvariance'
    kurtosis =      'kurtosis'
    skewness=       'skewness'


def dquote(pvname, dtype):
    if dtype == 'raw':
        return quote(pvname)
    else:
        # Is that name listed
        ApplianceOperators(dtype)
        return quote('{}({})'.format(dtype, pvname))

    raise AssertionError('dtype {} unknown'.format(dtype))


def read_header(line):
    header = proto.PayloadInfo()
    header.ParseFromString(decode(line))
    return header


def read_sequence(seq):
    # separate header line
    lines = seq.split(b'\n', 1)
    chunk = Chunk(read_header(bytearray(lines.pop(0))))
    # if existing read remaining data (here the main work is done)
    for line in lines:
        chunk.value = read_chunk(bytearray(line), chunk.header)
        #print('len = ',len(chunk.value))
    return chunk


@lru_cache(maxsize=64)
def get_data(data, *, return_type='pandas', time_format='timestamp',
             padding=False, t_start=None, t_stop=None):
    #print("get_data.cache_info: {}".format(request_data.cache_info()))
    """Parses HTTP/PB data buffer

    see [HTTPPB]_

    args:
        return_type (str, optional) : requested data type (pandas|raw). Defaults to pandas
        time_format (str, optional) : requested time format (raw, timestamp, datetime). Defaults to timestamp
        padding (str, optional) : restrict timestamp to requested time range (cuts first entry and adds dummy last entry)

    The following return types are supported
        'pandas'
            pandas DataFrame

        'raw'
            tuple (header, values, secs, nanos)

    The following time formats are supported:
        'raw'
           seconds (since beginning of year), nanoseconds

        'timestamp'
           seconds since 01.01.1970 (default)

        'datetime'
           datetime object ('pandas' only)


    Returns:
        tuple of numpy arrays or pandas.DataFrame

    Example::

        with open('data.pb','rb') as f:
            header, values, seconds, nanos = get_data(f.read(), return_type='raw')
    """

    res = []
    years = []
    # ignore last '\n' and split into chunks
    for seq in data[:-1].split(b'\n\n'):
        chunk = read_sequence(seq)
        # logger.debug('chunk header "{}"'.format(chunk.header))

        if chunk.value is not None:
            # logger.debug(chunk.value)
            res.append(chunk.value)
            header = chunk.header
            years.extend(chunk.header.year *
                         np.ones(len(chunk.value[0]), dtype=np.int))
            logger.debug(chunk.header)
            # print('found data:',len(chunk.value[0]))
        else:
            pass
        #    logger.debug('no data')

    # if single chunk with data, return here
    if len(res) == 0:
        logger.error('no data received')
        return None
    elif len(res) == 1:
        values, secs, nanos = res[0]
        years = years[0] * np.ones(len(secs), dtype=np.int)
        # print('One Chunk Only')
        # print('chunk.header.year = ',chunk.header.year)
    else:
        # if multible chunks, combine data and return
        # print("{} chunks found".format(len(res)))
        values = np.concatenate([r[0] for r in res])
        secs = np.concatenate([r[1] for r in res])
        nanos = np.concatenate([r[2] for r in res])

    if return_type == 'pandas':
        if time_format == 'datetime':
            # print('creating dataframe for year', header.year)
            df = pd.DataFrame({
                'year': years,
                'month': 1,
                'day': 1,
                'second': secs,
                'ns': nanos
            })
            dt = pd.to_datetime(df, utc=True)
            dt.name = 'datetime'
            df = pd.DataFrame(values, index=dt)
            if len(df.columns) == 1:
                df.columns = ['val']

            # Code of old versin
            df = df.tz_localize('UTC').tz_convert(dateutil.tz.tzlocal())
            # python3.7 pandas 0.24.2
            # df = df.tz_convert(dateutil.tz.tzlocal())

            if padding:
                if t_start is not None and t_stop is not None and len(df.columns) == 1:
                    # move first value to start of requested time range
                    first = pd.DataFrame([df['val'][0]], columns = ["val"], index=[t_start])
                    # add dummy last value at end of requested time range
                    last = pd.DataFrame([df['val'][-1]], columns = ["val"], index=[t_stop])
                    df = pd.concat([first ,df[1:], last ])
                else:
                    raise NotImplemented('padding not implemented for non-scalars')

        elif time_format == 'raw':
            df = pd.DataFrame({'second': secs, 'ns': nanos, 'val': values})
        elif time_format == 'timestamp':
            offset = datetime.datetime(header.year,
                                       1,
                                       1,
                                       tzinfo=dateutil.tz.tzutc()).timestamp()
            # df = pd.DataFrame({'timestamp':offset+secs+1.e-9*nanos, 'val':values})
            t = pd.Series(offset + secs + 1.e-9 * nanos, name='timestamp')
            df = pd.DataFrame(values, index=t)

        df.meta = {'header': header}

        return df
    else:  # return_type=='raw'
        return (header, values, secs, nanos)


class Archiver(ArchiverBasis):
    '''Access implemented with cython and protoc compiler
    '''
    def _getData(self, pvname, *, t0, t1, **kwargs):
        fmt = self.data_url_fmt
        url = fmt.format(format='raw', var=quote(pvname),
                         t0=quote(t0), t1=quote(t1))
        logger.debug('Using url %s', url)
        try:
            f = urlopen(url)
        except Exception as ex:
            logger.error('Failed to open url {}: reason {}'.format(url, ex))
            raise ex

        return get_data(f.read(), t_start=t0, t_stop=t1, **kwargs)

    @lru_cache(maxsize=64)
    def _requestData(self, pvname, *, t0, t1,  dtype='raw'):
        #print("request_data.cache_info: {}".format(request_data.cache_info()))
        fmt = self.data_url_fmt
        request = fmt.format(format='raw', var=dquote(pvname, dtype),
                             t0=quote(t0), t1=quote(t1))
        logger.info("request_data({}...)".format(request))
        try:
            f = urlopen(request)
            return f.read()
        except Exception as e:
            logger.error('Failed to handle request {} reason {}'.format(request, e))
        raise e

    def requestData(self, pvname, *, t0, t1, dtype='raw'):
        '''
        '''
        t0_str = convert_datetime_to_timestamp(t0)
        t1_str = convert_datetime_to_timestamp(t1)
        return self._requestData(pvname, t0=t0_str, t1=t1_str, dtype=dtype)

    def guessSize(self, pvname, t0, t1):

        try:
            info = self.getTypeInfo(pvname)
            has_type_info = True
        except HTTPError:
            logger.info('No Type Info')
            has_type_info = False

        if not has_type_info:
            data = self.requestData(pvname, t0=t0, t1=t0, dtype='firstSample')
            header, values, secs, nanos = get_data(data, return_type='raw')
            logger.debug(header)
            if header.type == 0 or header.type == 7:
                # TODO: support string types
                raise NotImplementedError('string types not supported yet')
            nbytes = np.typeDict[_dtypes[header.type]](0).nbytes
            nbytes *= header.elementCount
        else:
            count = int(info['elementCount'])
            dbrtype = info['DBRType'].split('_')[2]
            #print(dbrtype)
            if dbrtype in ['STRING']:
                # TODO: support string types
                raise NotImplementedError('string types not supported yet')
            else:
                nbytes = _dsize[_dbrtypes[dbrtype]] * count
                #print(count,nbytes)

        data = self.requestData(pvname, t0=t0, t1=t1, dtype='ncount')
        header, values, secs, nanos = get_data(data, return_type='raw')
        ncount = values[0]
        return (ncount, nbytes)



class DefaultArchiverNotFound(Exception):
    '''Default archiver was not found.

    Typically raised:

    The configuration file is read. In the default section the
    default archiver is named but no section in the configuration
    file uses the same name.
    '''



"""Access to archivers: code part

As user typically:

* get the source of bact-archiver-local
* rename it (e.g. for PSI bact-archiver-psi)
* edit config/archiver.cfg to reflect your achivers

Install your local package. This will install this package
in turn. See installation and configuration information in
packages documentation
"""

from .register import register_archivers

# No archivers here in this package
# Put this call in your local package
# register_archivers(__name__)



# -*- coding: utf-8 -*-
# -*- python -*-
# cython: language_level = 3str
# distutils: language = c++
# Author: Andreas Schälicke <andreas.schaelike@helmholtz-berlin.de>
# Date: 2017
"""Handling of EPICS archiver data

Documentation of the Archiver Protocol-buffer format:
    * https://slacmshankar.github.io/epicsarchiver_docs/pb_pbraw.html
    * https://developers.google.com/protocol-buffers/docs/cpptutorial


The following functions are expected to be called by external modules?
    * :func:`read_chunk`
    * :func:`read_header`
"""

# read EPICSEvent.pxd definition of Protocol-Buffer code
from proto.epics_event cimport PayloadInfo,  string
from proto.epics_event cimport ScalarDouble, ScalarString, ScalarEnum, ScalarInt
from proto.epics_event cimport VectorDouble, VectorFloat
from proto.epics_event cimport VectorInt, VectorShort, VectorChar

import numpy as np
cimport numpy as np
cimport cython



@cython.boundscheck(False) # turn off bounds-checking for entire function
@cython.wraparound(False)  # turn off negative index wrapping for entire function
cdef string cdecode(char[:] data, string & res) nogil:
    """Decode a PB message.

    Args:
        data : a char buffer

    Returns:
        string : a reference to the string to place the result into

    As serialized PB messages are binary data; after serialization, newline
    characters are escaped to maintain a "sample per line" constraint.

    ASCII codes are escaped in the following manner:
    * ``0x1B`` to  ``0x1B 0x01``
    * newline character ``\n`` =  ``0x0A`` to ``0x1B 0x02``
    * carriage return character ``0x0D`` to  ``0x1B 0x03``


    This encoding is reversed here

    WARNING:
           This function does not check that the string res is long enough!
    """
    cdef string *res_p = & res
    res_p.clear()
    cdef char c
    cdef int i=0
    cdef int N = data.shape[0]
    while i<N:
        c = data[i]
        if c==b'\x1b':
            i+=1
            c = data[i]
            if c==b'\x02':
                res_p[0]+=<char>b'\x0a'
            elif c==b'\x03':
                res_p[0]+=<char>b'\x0d'
            elif c==b'\x01':
                res_p[0]+=<char>b'\x1b'
        else:
            res_p[0]+=c
        i+=1

    return res


@cython.boundscheck(False)
@cython.wraparound(False)
cdef int count_lines(char[:] seq) nogil:
    """Number of new lines in the whole seq

    Args:
        seq: a character sequence

    Returns:
        count: position of new line

    New lines are defined by '\n'
    """
    cdef int N = seq.shape[0]
    cdef int i
    if N==0 :
        return 0
    cdef int count = 1
    for i in range(N):
        if seq[i] == '\n':
            count+=1
    return count

@cython.boundscheck(False)
@cython.wraparound(False)
cdef int cnext(char[:] seq, int idx) nogil:
    """find end of current line
    """
    cdef int N = seq.shape[0]
    cdef int i
    for i in range(idx,N):
        if seq[i] == '\n':
            return i
    return N

# reads one "chunk" according to the PB/HTTP protocol
# using defined "decoder" for the given EPICS type
# returning three numpy arrays

#
# ---------- SCALAR EPICS TYPES --------------------
#

# -0- EPICS STRING
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_chunk_str(char[:] seq, int N, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):

    values = np.empty(N,dtype=object)

    cdef ScalarString event

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf

    # needs GIL
    for i in range(N):
        start = stop+1
        stop = cnext(seq, start)
        event.ParseFromString(cdecode(seq[start:stop],buf))
        values[i] = event.val()
        secs[i] = event.secondsintoyear()
        nanos[i] = event.nano()

    return values,secs,nanos

# -3- EPICS ENUM
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_chunk_enum(char[:] seq, int N, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.int32_t] values = np.empty(N,dtype=np.int32)

    cdef ScalarEnum event

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            values[i] = event.val()
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()

    return values,secs,nanos

# -5- EPICS LONG
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_chunk_i4(char[:] seq, int N, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.int32_t] values = np.empty(N,dtype=np.int32)

    cdef ScalarInt event

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            values[i] = event.val()
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()

    return values,secs,nanos

# -6- EPICS DOUBLE - tested
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_chunk_f8(char[:] seq, int N, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.float_t] values = np.empty(N,dtype=np.float)

    cdef ScalarDouble event

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            values[i] = event.val()
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()

    return values,secs,nanos


#
# ---------- WAVEFORM EPICS TYPES --------------------
#

# -8- WAVEFORM SHORT - tested
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_vchunk_i2(char[:] seq, int N, int elements, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.int16_t, ndim=2] values = np.empty((N,elements),dtype=np.int16)

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf

    cdef VectorShort event
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            for j in range(elements):
                values[i,j] = event.val(j)
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()
    return values, secs, nanos


# -12- WAVEFORM LONG - failed?!
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_vchunk_i4(char[:] seq, int N, int elements, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.int32_t, ndim=2] values = np.empty((N,elements),dtype=np.int32)

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf

    cdef VectorInt event
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            for j in range(elements):
                values[i,j] = event.val(j)
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()
    return values, secs, nanos

# -9- WAVEFORM FLOAT - tested
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_vchunk_f4(char[:] seq, int N, int elements, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.float32_t, ndim=2] values = np.empty((N,elements),dtype=np.float32)

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf

    cdef VectorFloat event
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            for j in range(elements):
                values[i,j] = event.val(j)
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()
    return values, secs, nanos

# -13- WAVEFORM DOUBLE - tested
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_vchunk_f8(char[:] seq, int N, int elements, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.float64_t, ndim=2] values = np.empty((N,elements),dtype=np.float64)

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf

    cdef VectorDouble event
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            for j in range(elements):
                values[i,j] = event.val(j)
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()
    return values, secs, nanos

# -11- WAVEFORM CHAR
@cython.boundscheck(False)
@cython.wraparound(False)
cdef read_vchunk_char(char[:] seq, int N, int elements, np.ndarray[np.int32_t] secs, np.ndarray[np.int32_t] nanos):
    cdef np.ndarray[np.int8_t, ndim=2] values = np.empty((N,elements),dtype=np.int8)

    cdef int i
    cdef int start = 0
    cdef int stop = -1
    cdef string buf, val

    cdef VectorChar event
    with nogil:
        # should be save to release GIL here
        for i in range(N):
            start = stop+1
            stop = cnext(seq, start)
            event.ParseFromString(cdecode(seq[start:stop],buf))
            val = event.val()
            for j in range(elements):
                values[i,j] = val[j]
            secs[i] = event.secondsintoyear()
            nanos[i] = event.nano()
    return values, secs, nanos


#
# ---- PYTHON functions ----
#
def read_chunk(char[:] seq, header):
    """Read a protocol buffer chunk

    Args:
        seq : a character buffer
        epics_type : an integer defining the epics type
        elements   : the number of elements


    Returns:
      values, seconds, nano_seconds


    Currently the following epics types are implemented:
        *  0.  : string
        *  3.  : enum
        *  5.  : i4 (signed integers of four bytes)
        *  6.  : f8 (doubles of eight bytes)
        *  9.  : i2 vector (signed [short] integers of two bytes)
        *  11. : vector of characters
        *  12. : f4 vector (floats of four bytes)
        *  13. : f8 vector (floats of eight bytes)

    """
    cdef int epics_type = header.type
    cdef int elements = header.elementCount
    cdef int N = count_lines(seq)

    cdef np.ndarray[np.int32_t] secs = np.empty(N,dtype=np.int32)
    cdef np.ndarray[np.int32_t] nanos = np.empty(N,dtype=np.int32)

    if epics_type==0:
        return read_chunk_str(seq, N, secs, nanos)
    elif epics_type==3:
        return read_chunk_enum(seq, N, secs, nanos)
    elif epics_type==5:
        return read_chunk_i4(seq, N, secs, nanos)
    elif epics_type==6:
        return read_chunk_f8(seq, N, secs, nanos)
    elif epics_type==8:
        return read_vchunk_i2(seq, N, elements, secs, nanos)
    elif epics_type==9:
        return read_vchunk_f4(seq, N, elements, secs, nanos)
    elif epics_type==11:
        return read_vchunk_char(seq, N, elements, secs, nanos)
    elif epics_type==12:
        return read_vchunk_i4(seq, N, elements, secs, nanos)
    elif epics_type==13:
        return read_vchunk_f8(seq, N, elements, secs, nanos)

    # Why not raise an exception here
    raise NotImplementedError('Type {} not supported'.format(epics_type))


def decode(char[:] line):
    """Simple python wraper for cdecode
    """
    cdef string buf
    cdecode(line,buf)
    return buf

def read_header(char[:] line):
    """Read the header of the payload

    Args:
        line : char buffer

    Returns:
        dic: dictionary containing
            * type
            * year
            * elementcount

    use carchiver.read_header if PayloadInfo is need directly
    """
    cdef PayloadInfo info = PayloadInfo()
    cdef string buf
    info.ParseFromString(cdecode(line,buf))

    header = {}
    header['type'] = info.type()
    header['year'] = info.year()
    header['elementcount'] = info.elementcount()

    return header



# -*- coding: utf-8 -*-
# Author: Andreas Schaelicke <andreas.schaelicke@helmholtz-berlin.de>
#         Pierre Schnizer <pierre.schnizer@helmholtz-berlin.de>
# Time-stamp:
"""Archiver interface pythonic version

"""
from .archiver import ArchiverBasis
from .protocol_buffer import Chunk, dtypes as _dtypes, decoder as _decoder
from . import epics_event_pb2 as proto

from urllib.request import urlopen, quote
import numpy as np


class Archiver(ArchiverBasis):
    def _getData(self, pvname, *, t0, t1, **kwargs):
        fmt = self.data_url_fmt
        url = fmt.format(format='raw', var=quote(pvname),
                         t0=quote(t0), t1=quote(t1))
        f = urlopen(url)
        data = get_data(f.read())
        return data


def get_data(data):
    chunks = []
    for seq in data.split(b'\n\n'):
        lines = seq.splitlines()

        header = proto.PayloadInfo()
        header.ParseFromString(
            lines.pop(0).replace(b'\x1b\x02', b'\x0a').replace(
                b'\x1b\x03', b'\x0d').replace(b'\x1b\x01', b'\x1b'))
        chunk = Chunk(header)
        chunks.append(chunk)

        dtype = _dtypes[header.type]

        if 7 <= header.type <= 13:
            # vector decoder
            def parse_vec(inp, event=_decoder[header.type]()):
                event.ParseFromString(
                    inp.replace(b'\x1b\x02', b'\x0a').replace(
                        b'\x1b\x03', b'\x0d').replace(b'\x1b\x01', b'\x1b'))
                return [v
                        for v in event.val], event.secondsintoyear, event.nano

            chunk.values = np.array(
                [parse_vec(l) for l in lines],
                dtype=[('value', dtype, (header.elementCount, )),
                       ('sec', 'u4'), ('ns', 'u4')])
        else:
            # scalar decoder
            def parse(inp, event=_decoder[header.type]()):
                event.ParseFromString(
                    inp.replace(b'\x1b\x02', b'\x0a').replace(
                        b'\x1b\x03', b'\x0d').replace(b'\x1b\x01', b'\x1b'))
                return event.val, event.secondsintoyear, event.nano

            chunk.values = np.array(
                [parse(l) for l in lines],
                dtype=[('value', dtype), ('sec', 'u4'), ('ns', 'u4')])

    return header, np.concatenate([c.values for c in chunks])



# distutils: language = c++
# cython: language_level = 2


from libcpp.string cimport string
from libcpp cimport bool

cdef extern from "google/protobuf/stubs/port.h" namespace "::google::protobuf":
     ctypedef int int32
     ctypedef int uint32

cdef extern from "epics_event.pb.h" namespace "EPICS":
    cdef enum PayloadType:
        pass

    cdef cppclass FieldValue:
        pass

    cdef cppclass PayloadInfo nogil:
        PayloadInfo() except +
        bool ParseFromString(const string& data) except +
        int32 year()
        PayloadType type()
        int32 elementcount()
        string pvname()
        int32 headers_size()
        FieldValue headers(int)

    # LINAC1C:stGun
    cdef cppclass ScalarString nogil:
        ScalarString() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        string val()

    # TOPUPCC:rdCur
    cdef cppclass ScalarDouble nogil:
        ScalarDouble() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        float val()

    # TOPUPCC:numShots
    cdef cppclass ScalarInt nogil:
        ScalarInt() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        int32 val()

    # TOPUPCC:selTrgSR
    cdef cppclass ScalarEnum nogil:
        ScalarEnum() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        int32 val()

    # CUMZR:MBcurrent
    cdef cppclass VectorDouble nogil:
        VectorDouble() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        float val(int)

    # BBQR:X:SRAM:MEAN
    cdef cppclass VectorFloat nogil:
        VectorFloat() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        float val(int)

    # BBQR:X:FB:MASK
    cdef cppclass VectorChar nogil:
        VectorChar() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        string val()

    # BBQR:X:SRAM:MAXRMS
    cdef cppclass VectorShort nogil:
        VectorShort() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        int32 val(int)

    # FILL:fullhistogram
    cdef cppclass VectorInt nogil:
        VectorInt() except +
        bool ParseFromString(const string& data) except +
        uint32 secondsintoyear()
        uint32 nano()
        int32 val(int)



'''Access to the epics archiver appliance

Typical usage:
     * Download package bact-archiver-local
     * Edit the archiver.cfg package there to reflect your installation
     * Installing this package will automatically install this package
'''

from abc import ABCMeta, abstractmethod, abstractproperty
import json
import logging
from urllib.request import urlopen, quote


logger = logging.getLogger('bact-archiver')


#: The format as requested by the archiver
archiver_request_fmt = "%Y-%m-%dT%H:%M:%S.000000Z"


def convert_datetime_to_timestamp(datum):
    '''Convert datetime to string as expected by archiver

    Args:
        datum: a :class:`datetime.datetime` object
    '''
    try:
        r = datum.strftime(archiver_request_fmt)
    except Exception:
        logger.error('Failed converting %s of type %s to timestamp string',
                     datum, type(datum))
        raise
    return r


class ArchiverInterface(metaclass=ABCMeta):
    '''Archiver interface definition
    '''
    @abstractproperty
    def name(self):
        'Name of the archiver'

    @abstractproperty
    def description(self):
        'Description of the archiver'

    @abstractmethod
    def getData(self, var, * t0, t1, **kws):
        """Get archiver data for single EPICS variable in given time frame.

        Args:
            t0 :                         start time a :class:`datetime.datetime` object
            t1 :                         end time a :class:`datetime.datetime` object

            return_type (str, optional) : requested data type (pandas|raw).
                                          Defaults to pandas
            time_format (str, optional) : requested time format (raw, timestamp, datetime).
                                          Defaults to timestamp
            padding (str, optional) :    restrict timestamp to requested time range
                                         (cuts first entry and adds dummy last entry)

        Returns:
            tuple of numpy arrays or pandas.DataFrame see :func:`get_data`

        The following return types are supported
            'pandas'
                a :class:`pandas.DataFrame`

            'raw'
                tuple (header, values, secs, nanos)

        The following time formats are supported:
            'raw'
               seconds (since beginning of year), nanoseconds

            'timestamp'
               seconds since 01.01.1970 (default)

            'datetime'
               datetime object ('pandas' only)

        Example::

            t0 = datetime.datetime(2017, 10, 02, 21)
            t1 = datetime.datetime(2017, 10, 02, 21, 5)
            df = archiver.getData('TOPUPCC:rdCur', t0=t0, t1=t1,
                                  return_type='pandas', time_format='datetime')
            print(df.meta['header'])
            plot(df.index, df.values.flatten())
        """
        raise NotImplementedError('Implement in derived class')

    # def __call__(self, t0, t1, **kwargs):
    #    "I am not supporting to implement this method"
    #    return self.getData(t0, t1, **kwargs)


class ArchiverBasis(ArchiverInterface):
    '''

    Warning:
        Work in progress

    Used for the python or the carchiver
    '''
    def __init__(self, *, config):
        self.config = config

    @property
    def name(self):
        return self.config.name

    @property
    def description(self):
        return self.config.description

    @property
    def data_url_fmt(self):
        url = self.config.retrieval_url
        url += "/data/getData.{format}?pv={var}&from={t0}&to={t1}&ca_how=0"
        return url

    @property
    def bpl_url_fmt(self):
        '''

        Todo: better naming?
        '''
        url = self.config.retrieval_url
        url += '/bpl/{cmd}{opt}'
        return url

    def getData(self, pvname, *, t0, t1, **kws):
        t0_str = convert_datetime_to_timestamp(t0)
        t1_str = convert_datetime_to_timestamp(t1)
        fmt = 'Trying to get data for pv %s in interval %s..%s = %s..%s',
        logger.info(fmt, pvname, t0, t1, t0_str, t1_str)

        return self._getData(pvname, t0=t0_str, t1=t1_str, **kws)

    def askAppliance(self, cmd, **kwargs):
        '''
        '''
        # cmds :
        opts = ''
        opt = '?{}={}'
        for k, v in kwargs.items():
            opts += opt.format(k, v)
            opt = '&{}={}'

        fmt = self.bpl_url_fmt
        url = fmt.format(cmd=cmd, opt=opts)
        fmt = 'Asking archiver %s  using url %s'

        logger.info(fmt, self.name, url)
        try:
            request = urlopen(url)
        except Exception as ex:
            logger.error(fmt + ' Reason %s', self.name, url, ex)
            raise ex

        data = json.loads(request.read().decode('UTF-8'))
        return data

    def getAllPVs(self):
        return self.getMatchingPVs()

    def getMatchingPVs(self, pv="*"):
        return self.askAppliance('getMatchingPVs', pv=pv)

    def getTypeInfo(self, pv):
        return self.askAppliance('getMetadata', pv=pv)

    def __repr__(self):
        args_text = 'config={}'.format(self.config)
        txt = "{}({}, {})".format(self.__class__.__name__, self.name,
                                  args_text)
        return txt

    def saveBPRaw(self, pvname, *,  t0, t1, fname='test.pb'):
        fmt = self.data_url_fmt
        url = fmt.format(format='raw', var=quote(pvname),
                         t0=quote(t0), t1=quote(t1))
        f = urlopen(url)

        with open(fname, 'wb') as fout:
            fout.write(f.read())


def save_hdf5(data, *, fname=None):
    import h5py
    chunk = data.chunks[1]
    values = data.values

    with h5py.File(fname, 'w') as f:
        group = f.create_group(chunk.header.pvname)
        group.create_dataset(
            'value', data=values['value'], compression='gzip', shuffle=True)
        group.create_dataset(
            'sec', data=values['sec'], compression='gzip', shuffle=True)
        group.create_dataset(
            'ns', data=values['ns'], compression='gzip', shuffle=True)



# -*- coding: utf-8 -*-
# Author: Andreas Schälicke <andreas.schaelicke@helmholtz-berlin.de>
#          Pierre Schnizer <pierre.schnizer@helmholtz-berlin.de>
# Date: 2017, 2020
"""Generate the interface code for Google Protobuf

For Google protocol buffer see
https://developers.google.com/protocol-buffers
"""

import os.path
import setuptools
import logging

logger = logging.getLogger('setup')
_log = setuptools.distutils.log

class GenerateProtocolBuffer(setuptools.Command):
    """Custom build step for protobuf

    Generate the python and c++ wrapper code from the protocol buffer specifications
    """

    description = "Run protoc to generate python and cpp wrapper files"

    user_options = [
        # The format is (long option, short option, description).
        ('inplace', None, 'create files inplace of proto-file (default)'),
        ('python', None, 'create python wrapper'),
        ('cpp', None, 'create C++ wrapper'),
        ('source', None, 'files to be processed by protoc'),
        ('src-dir', None, 'directory, where the input files are located'),
        ('pydir', None, 'directroy, where the python output will be placed'),
        ('build-dir', None, 'directroy, where the output will be placed'),
        ('protoc=', None, 'protoc executable to use'),
    ]

    def initialize_options(self):
        """Set default values for options."""
        # Each user option must be listed here with their default value.
        self.inplace = True
        self.python = False
        self.cpp = False
        self.protoc = None
        cwd = os.getcwd()
        self.source = ''
        self.pydir = cwd
        self.src_dir = cwd
        self.build_dir = cwd


    def finalize_options(self):
        """Post-process options."""
        if not self.inplace:
            self.announce('inplace False is not support yet',
                          level=_log.WARN)

        if not self.python and not self.cpp:
            self.python = True
            self.cpp = True
            self.announce('select python and C++ wrapper',
                          level=_log.INFO)

    def run(self):
        self.announce("creating Wrapper for Archiver Protocol Buffers",
                      level=_log.INFO)

        if self.protoc is None:
            protoc = 'protoc'
        else:
            protoc = self.protoc

        #self.announce('using protoc "%s"' %(protoc,), level=_log.INFO)
        args = [protoc, self.source, '--proto_path={}'.format(self.src_dir)]

        if self.cpp:
            self.spawn(args + ['--cpp_out={}'.format(self.build_dir)])
        if self.python:
            self.spawn(args + ['--python_out={}'.format(self.pydir)])



'''Support functions for adding archivers to the module
'''
import sys
import logging
from .config import archiver_configurations
from .pyarchiver import Archiver
from .carchiver import Archiver

logger = logging.getLogger('bact-archiver')


def add_archivers_to_module(mod_name, configs):
    '''
    Args:
        mod_name: the name of the module to which the archivers
                  should be added.
        configs:  sequencee of archiver configurations

    each archiver configuration should conform to the interface
    defined by
    :class:`bact_archiver.config,ArchiverConfigurationInterface`
    '''

    t_mod = sys.modules[mod_name]
    logger.info('Adding archivers %s to mod %s', list(configs.keys()), t_mod)

    archivers = {tup[0]: Archiver(config=tup[1]) for tup in configs.items()}
    setattr(t_mod, 'archivers', archivers)
    for name, ac in archivers.items():
        setattr(t_mod, name, ac)


def register_archivers(module_name):
    '''Register the archivers to the module
    '''
    configs = archiver_configurations(module_name)
    add_archivers_to_module(module_name, configs)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: epics_event.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='epics_event.proto',
  package='EPICS',
  syntax='proto2',
  serialized_options=_b('\n&edu.stanford.slac.archiverappliance.PBB\nEPICSEvent'),
  serialized_pb=_b('\n\x11\x65pics_event.proto\x12\x05\x45PICS\"\'\n\nFieldValue\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0b\n\x03val\x18\x02 \x02(\t\"\xc2\x01\n\x0cScalarString\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\t\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc0\x01\n\nScalarByte\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x0c\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc1\x01\n\x0bScalarShort\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x11\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xbf\x01\n\tScalarInt\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x0f\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc0\x01\n\nScalarEnum\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x11\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc1\x01\n\x0bScalarFloat\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x02\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc2\x01\n\x0cScalarDouble\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc2\x01\n\x0cVectorString\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x03(\t\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc0\x01\n\nVectorChar\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x0c\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc5\x01\n\x0bVectorShort\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0f\n\x03val\x18\x03 \x03(\x11\x42\x02\x10\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc3\x01\n\tVectorInt\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0f\n\x03val\x18\x03 \x03(\x0f\x42\x02\x10\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc4\x01\n\nVectorEnum\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0f\n\x03val\x18\x03 \x03(\x11\x42\x02\x10\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc5\x01\n\x0bVectorFloat\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0f\n\x03val\x18\x03 \x03(\x02\x42\x02\x10\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xc6\x01\n\x0cVectorDouble\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0f\n\x03val\x18\x03 \x03(\x01\x42\x02\x10\x01\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\"\xd5\x01\n\x0eV4GenericBytes\x12\x17\n\x0fsecondsintoyear\x18\x01 \x02(\r\x12\x0c\n\x04nano\x18\x02 \x02(\r\x12\x0b\n\x03val\x18\x03 \x02(\x0c\x12\x13\n\x08severity\x18\x04 \x01(\x05:\x01\x30\x12\x11\n\x06status\x18\x05 \x01(\x05:\x01\x30\x12\x13\n\x0brepeatcount\x18\x06 \x01(\r\x12&\n\x0b\x66ieldvalues\x18\x07 \x03(\x0b\x32\x11.EPICS.FieldValue\x12\x19\n\x11\x66ieldactualchange\x18\x08 \x01(\x08\x12\x0f\n\x07userTag\x18\t \x01(\r\"\xbb\x02\n\x0bPayloadInfo\x12 \n\x04type\x18\x01 \x02(\x0e\x32\x12.EPICS.PayloadType\x12\x0e\n\x06pvname\x18\x02 \x02(\t\x12\x0c\n\x04year\x18\x03 \x02(\x05\x12\x14\n\x0c\x65lementCount\x18\x04 \x01(\x05\x12\x10\n\x08unused00\x18\x05 \x01(\x01\x12\x10\n\x08unused01\x18\x06 \x01(\x01\x12\x10\n\x08unused02\x18\x07 \x01(\x01\x12\x10\n\x08unused03\x18\x08 \x01(\x01\x12\x10\n\x08unused04\x18\t \x01(\x01\x12\x10\n\x08unused05\x18\n \x01(\x01\x12\x10\n\x08unused06\x18\x0b \x01(\x01\x12\x10\n\x08unused07\x18\x0c \x01(\x01\x12\x10\n\x08unused08\x18\r \x01(\x01\x12\x10\n\x08unused09\x18\x0e \x01(\t\x12\"\n\x07headers\x18\x0f \x03(\x0b\x32\x11.EPICS.FieldValue*\xa9\x02\n\x0bPayloadType\x12\x11\n\rSCALAR_STRING\x10\x00\x12\x10\n\x0cSCALAR_SHORT\x10\x01\x12\x10\n\x0cSCALAR_FLOAT\x10\x02\x12\x0f\n\x0bSCALAR_ENUM\x10\x03\x12\x0f\n\x0bSCALAR_BYTE\x10\x04\x12\x0e\n\nSCALAR_INT\x10\x05\x12\x11\n\rSCALAR_DOUBLE\x10\x06\x12\x13\n\x0fWAVEFORM_STRING\x10\x07\x12\x12\n\x0eWAVEFORM_SHORT\x10\x08\x12\x12\n\x0eWAVEFORM_FLOAT\x10\t\x12\x11\n\rWAVEFORM_ENUM\x10\n\x12\x11\n\rWAVEFORM_BYTE\x10\x0b\x12\x10\n\x0cWAVEFORM_INT\x10\x0c\x12\x13\n\x0fWAVEFORM_DOUBLE\x10\r\x12\x14\n\x10V4_GENERIC_BYTES\x10\x0e\x42\x34\n&edu.stanford.slac.archiverappliance.PBB\nEPICSEvent')
)

_PAYLOADTYPE = _descriptor.EnumDescriptor(
  name='PayloadType',
  full_name='EPICS.PayloadType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SCALAR_STRING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_SHORT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_FLOAT', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_ENUM', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_BYTE', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_INT', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCALAR_DOUBLE', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_STRING', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_SHORT', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_FLOAT', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_ENUM', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_BYTE', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_INT', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAVEFORM_DOUBLE', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='V4_GENERIC_BYTES', index=14, number=14,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3364,
  serialized_end=3661,
)
_sym_db.RegisterEnumDescriptor(_PAYLOADTYPE)

PayloadType = enum_type_wrapper.EnumTypeWrapper(_PAYLOADTYPE)
SCALAR_STRING = 0
SCALAR_SHORT = 1
SCALAR_FLOAT = 2
SCALAR_ENUM = 3
SCALAR_BYTE = 4
SCALAR_INT = 5
SCALAR_DOUBLE = 6
WAVEFORM_STRING = 7
WAVEFORM_SHORT = 8
WAVEFORM_FLOAT = 9
WAVEFORM_ENUM = 10
WAVEFORM_BYTE = 11
WAVEFORM_INT = 12
WAVEFORM_DOUBLE = 13
V4_GENERIC_BYTES = 14



_FIELDVALUE = _descriptor.Descriptor(
  name='FieldValue',
  full_name='EPICS.FieldValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='EPICS.FieldValue.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.FieldValue.val', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=67,
)


_SCALARSTRING = _descriptor.Descriptor(
  name='ScalarString',
  full_name='EPICS.ScalarString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarString.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarString.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarString.val', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarString.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarString.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarString.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarString.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarString.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=70,
  serialized_end=264,
)


_SCALARBYTE = _descriptor.Descriptor(
  name='ScalarByte',
  full_name='EPICS.ScalarByte',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarByte.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarByte.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarByte.val', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarByte.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarByte.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarByte.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarByte.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarByte.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=267,
  serialized_end=459,
)


_SCALARSHORT = _descriptor.Descriptor(
  name='ScalarShort',
  full_name='EPICS.ScalarShort',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarShort.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarShort.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarShort.val', index=2,
      number=3, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarShort.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarShort.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarShort.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarShort.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarShort.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=462,
  serialized_end=655,
)


_SCALARINT = _descriptor.Descriptor(
  name='ScalarInt',
  full_name='EPICS.ScalarInt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarInt.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarInt.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarInt.val', index=2,
      number=3, type=15, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarInt.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarInt.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarInt.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarInt.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarInt.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=849,
)


_SCALARENUM = _descriptor.Descriptor(
  name='ScalarEnum',
  full_name='EPICS.ScalarEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarEnum.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarEnum.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarEnum.val', index=2,
      number=3, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarEnum.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarEnum.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarEnum.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarEnum.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarEnum.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=852,
  serialized_end=1044,
)


_SCALARFLOAT = _descriptor.Descriptor(
  name='ScalarFloat',
  full_name='EPICS.ScalarFloat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarFloat.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarFloat.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarFloat.val', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarFloat.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarFloat.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarFloat.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarFloat.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarFloat.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1047,
  serialized_end=1240,
)


_SCALARDOUBLE = _descriptor.Descriptor(
  name='ScalarDouble',
  full_name='EPICS.ScalarDouble',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.ScalarDouble.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.ScalarDouble.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.ScalarDouble.val', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.ScalarDouble.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.ScalarDouble.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.ScalarDouble.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.ScalarDouble.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.ScalarDouble.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1243,
  serialized_end=1437,
)


_VECTORSTRING = _descriptor.Descriptor(
  name='VectorString',
  full_name='EPICS.VectorString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorString.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorString.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorString.val', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorString.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorString.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorString.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorString.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorString.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1440,
  serialized_end=1634,
)


_VECTORCHAR = _descriptor.Descriptor(
  name='VectorChar',
  full_name='EPICS.VectorChar',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorChar.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorChar.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorChar.val', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorChar.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorChar.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorChar.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorChar.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorChar.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1637,
  serialized_end=1829,
)


_VECTORSHORT = _descriptor.Descriptor(
  name='VectorShort',
  full_name='EPICS.VectorShort',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorShort.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorShort.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorShort.val', index=2,
      number=3, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorShort.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorShort.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorShort.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorShort.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorShort.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1832,
  serialized_end=2029,
)


_VECTORINT = _descriptor.Descriptor(
  name='VectorInt',
  full_name='EPICS.VectorInt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorInt.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorInt.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorInt.val', index=2,
      number=3, type=15, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorInt.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorInt.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorInt.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorInt.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorInt.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2032,
  serialized_end=2227,
)


_VECTORENUM = _descriptor.Descriptor(
  name='VectorEnum',
  full_name='EPICS.VectorEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorEnum.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorEnum.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorEnum.val', index=2,
      number=3, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorEnum.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorEnum.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorEnum.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorEnum.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorEnum.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2230,
  serialized_end=2426,
)


_VECTORFLOAT = _descriptor.Descriptor(
  name='VectorFloat',
  full_name='EPICS.VectorFloat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorFloat.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorFloat.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorFloat.val', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorFloat.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorFloat.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorFloat.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorFloat.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorFloat.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2429,
  serialized_end=2626,
)


_VECTORDOUBLE = _descriptor.Descriptor(
  name='VectorDouble',
  full_name='EPICS.VectorDouble',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.VectorDouble.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.VectorDouble.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.VectorDouble.val', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.VectorDouble.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.VectorDouble.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.VectorDouble.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.VectorDouble.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.VectorDouble.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2629,
  serialized_end=2827,
)


_V4GENERICBYTES = _descriptor.Descriptor(
  name='V4GenericBytes',
  full_name='EPICS.V4GenericBytes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='secondsintoyear', full_name='EPICS.V4GenericBytes.secondsintoyear', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nano', full_name='EPICS.V4GenericBytes.nano', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='EPICS.V4GenericBytes.val', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='EPICS.V4GenericBytes.severity', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='EPICS.V4GenericBytes.status', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repeatcount', full_name='EPICS.V4GenericBytes.repeatcount', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldvalues', full_name='EPICS.V4GenericBytes.fieldvalues', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fieldactualchange', full_name='EPICS.V4GenericBytes.fieldactualchange', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userTag', full_name='EPICS.V4GenericBytes.userTag', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2830,
  serialized_end=3043,
)


_PAYLOADINFO = _descriptor.Descriptor(
  name='PayloadInfo',
  full_name='EPICS.PayloadInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='EPICS.PayloadInfo.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pvname', full_name='EPICS.PayloadInfo.pvname', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='year', full_name='EPICS.PayloadInfo.year', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='elementCount', full_name='EPICS.PayloadInfo.elementCount', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused00', full_name='EPICS.PayloadInfo.unused00', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused01', full_name='EPICS.PayloadInfo.unused01', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused02', full_name='EPICS.PayloadInfo.unused02', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused03', full_name='EPICS.PayloadInfo.unused03', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused04', full_name='EPICS.PayloadInfo.unused04', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused05', full_name='EPICS.PayloadInfo.unused05', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused06', full_name='EPICS.PayloadInfo.unused06', index=10,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused07', full_name='EPICS.PayloadInfo.unused07', index=11,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused08', full_name='EPICS.PayloadInfo.unused08', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unused09', full_name='EPICS.PayloadInfo.unused09', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='headers', full_name='EPICS.PayloadInfo.headers', index=14,
      number=15, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3046,
  serialized_end=3361,
)

_SCALARSTRING.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARBYTE.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARSHORT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARINT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARENUM.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARFLOAT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_SCALARDOUBLE.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORSTRING.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORCHAR.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORSHORT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORINT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORENUM.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORFLOAT.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_VECTORDOUBLE.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_V4GENERICBYTES.fields_by_name['fieldvalues'].message_type = _FIELDVALUE
_PAYLOADINFO.fields_by_name['type'].enum_type = _PAYLOADTYPE
_PAYLOADINFO.fields_by_name['headers'].message_type = _FIELDVALUE
DESCRIPTOR.message_types_by_name['FieldValue'] = _FIELDVALUE
DESCRIPTOR.message_types_by_name['ScalarString'] = _SCALARSTRING
DESCRIPTOR.message_types_by_name['ScalarByte'] = _SCALARBYTE
DESCRIPTOR.message_types_by_name['ScalarShort'] = _SCALARSHORT
DESCRIPTOR.message_types_by_name['ScalarInt'] = _SCALARINT
DESCRIPTOR.message_types_by_name['ScalarEnum'] = _SCALARENUM
DESCRIPTOR.message_types_by_name['ScalarFloat'] = _SCALARFLOAT
DESCRIPTOR.message_types_by_name['ScalarDouble'] = _SCALARDOUBLE
DESCRIPTOR.message_types_by_name['VectorString'] = _VECTORSTRING
DESCRIPTOR.message_types_by_name['VectorChar'] = _VECTORCHAR
DESCRIPTOR.message_types_by_name['VectorShort'] = _VECTORSHORT
DESCRIPTOR.message_types_by_name['VectorInt'] = _VECTORINT
DESCRIPTOR.message_types_by_name['VectorEnum'] = _VECTORENUM
DESCRIPTOR.message_types_by_name['VectorFloat'] = _VECTORFLOAT
DESCRIPTOR.message_types_by_name['VectorDouble'] = _VECTORDOUBLE
DESCRIPTOR.message_types_by_name['V4GenericBytes'] = _V4GENERICBYTES
DESCRIPTOR.message_types_by_name['PayloadInfo'] = _PAYLOADINFO
DESCRIPTOR.enum_types_by_name['PayloadType'] = _PAYLOADTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FieldValue = _reflection.GeneratedProtocolMessageType('FieldValue', (_message.Message,), dict(
  DESCRIPTOR = _FIELDVALUE,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.FieldValue)
  ))
_sym_db.RegisterMessage(FieldValue)

ScalarString = _reflection.GeneratedProtocolMessageType('ScalarString', (_message.Message,), dict(
  DESCRIPTOR = _SCALARSTRING,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarString)
  ))
_sym_db.RegisterMessage(ScalarString)

ScalarByte = _reflection.GeneratedProtocolMessageType('ScalarByte', (_message.Message,), dict(
  DESCRIPTOR = _SCALARBYTE,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarByte)
  ))
_sym_db.RegisterMessage(ScalarByte)

ScalarShort = _reflection.GeneratedProtocolMessageType('ScalarShort', (_message.Message,), dict(
  DESCRIPTOR = _SCALARSHORT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarShort)
  ))
_sym_db.RegisterMessage(ScalarShort)

ScalarInt = _reflection.GeneratedProtocolMessageType('ScalarInt', (_message.Message,), dict(
  DESCRIPTOR = _SCALARINT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarInt)
  ))
_sym_db.RegisterMessage(ScalarInt)

ScalarEnum = _reflection.GeneratedProtocolMessageType('ScalarEnum', (_message.Message,), dict(
  DESCRIPTOR = _SCALARENUM,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarEnum)
  ))
_sym_db.RegisterMessage(ScalarEnum)

ScalarFloat = _reflection.GeneratedProtocolMessageType('ScalarFloat', (_message.Message,), dict(
  DESCRIPTOR = _SCALARFLOAT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarFloat)
  ))
_sym_db.RegisterMessage(ScalarFloat)

ScalarDouble = _reflection.GeneratedProtocolMessageType('ScalarDouble', (_message.Message,), dict(
  DESCRIPTOR = _SCALARDOUBLE,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.ScalarDouble)
  ))
_sym_db.RegisterMessage(ScalarDouble)

VectorString = _reflection.GeneratedProtocolMessageType('VectorString', (_message.Message,), dict(
  DESCRIPTOR = _VECTORSTRING,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorString)
  ))
_sym_db.RegisterMessage(VectorString)

VectorChar = _reflection.GeneratedProtocolMessageType('VectorChar', (_message.Message,), dict(
  DESCRIPTOR = _VECTORCHAR,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorChar)
  ))
_sym_db.RegisterMessage(VectorChar)

VectorShort = _reflection.GeneratedProtocolMessageType('VectorShort', (_message.Message,), dict(
  DESCRIPTOR = _VECTORSHORT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorShort)
  ))
_sym_db.RegisterMessage(VectorShort)

VectorInt = _reflection.GeneratedProtocolMessageType('VectorInt', (_message.Message,), dict(
  DESCRIPTOR = _VECTORINT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorInt)
  ))
_sym_db.RegisterMessage(VectorInt)

VectorEnum = _reflection.GeneratedProtocolMessageType('VectorEnum', (_message.Message,), dict(
  DESCRIPTOR = _VECTORENUM,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorEnum)
  ))
_sym_db.RegisterMessage(VectorEnum)

VectorFloat = _reflection.GeneratedProtocolMessageType('VectorFloat', (_message.Message,), dict(
  DESCRIPTOR = _VECTORFLOAT,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorFloat)
  ))
_sym_db.RegisterMessage(VectorFloat)

VectorDouble = _reflection.GeneratedProtocolMessageType('VectorDouble', (_message.Message,), dict(
  DESCRIPTOR = _VECTORDOUBLE,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.VectorDouble)
  ))
_sym_db.RegisterMessage(VectorDouble)

V4GenericBytes = _reflection.GeneratedProtocolMessageType('V4GenericBytes', (_message.Message,), dict(
  DESCRIPTOR = _V4GENERICBYTES,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.V4GenericBytes)
  ))
_sym_db.RegisterMessage(V4GenericBytes)

PayloadInfo = _reflection.GeneratedProtocolMessageType('PayloadInfo', (_message.Message,), dict(
  DESCRIPTOR = _PAYLOADINFO,
  __module__ = 'epics_event_pb2'
  # @@protoc_insertion_point(class_scope:EPICS.PayloadInfo)
  ))
_sym_db.RegisterMessage(PayloadInfo)


DESCRIPTOR._options = None
_VECTORSHORT.fields_by_name['val']._options = None
_VECTORINT.fields_by_name['val']._options = None
_VECTORENUM.fields_by_name['val']._options = None
_VECTORFLOAT.fields_by_name['val']._options = None
_VECTORDOUBLE.fields_by_name['val']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Authors : Andreas Schälicke <andreas.schaelike@helmholtz-berlin.de>
#           Pierre Schnizer <pierre.schnizer@helmholtz-berlin.de>
# Date : 2017, 2020

from cysetuptools import setup
import protocol_buffer

cmdclass = dict(build_proto_c=protocol_buffer.GenerateProtocolBuffer)
setup(cmdclass=cmdclass,
      author = "Andreas Schälicke, Pierre Schnizer",
      author_email = ("andreas.schälicke@helmholtz-berlin.de, "
                      "pierre.schnizer@helmholtz-berlin.de")
)



import subprocess
import os
import os.path as op
import sys
import shlex
import argparse

import setuptools

PY3 = sys.version_info[0] == 3
if PY3:
    import configparser
else:
    import ConfigParser as configparser


DEFAULTS_SECTION = 'cython-defaults'
MODULE_SECTION_PREFIX = 'cython-module:'
CYTHON_EXT = '.pyx'
C_EXT = '.c'
CPP_EXT = '.cpp'


def setup(cythonize=True, **kwargs):
    """
    Drop-in replacement for :func:`setuptools.setup`, adding Cython niceties.

    Cython modules are described in setup.cfg, for example::

        [cython-module: foo.bar]
        sources = foo.pyx
                  bar.cpp
        include_dirs = eval(__import__('numpy').get_include())
                       /usr/include/foo
        language = c++
        pkg_config_packages = opencv

    You still need to provide a ``setup.py``::

        from cysetuptools import setup

        setup()

    The modules sections support the following entries:

    sources
        The list of Cython and C/C++ source files that are compiled to build
        the module.

    libraries
        A list of libraries to link with the module.

    include_dirs
        A list of directories to find include files. This entry supports
        python expressions with ``eval()``; in the example above this is used
        to retrieve the numpy include directory.

    library_dirs
        A list of directories to find libraries. This entry supports
        python expressions with ``eval()`` like ``include_dirs``.

    extra_compile_args
        Extra arguments passed to the compiler.

    extra_link_args
        Extra arguments passed to the linker.

    language
        Typically "c" or "c++".

    pkg_config_packages
        A list of ``pkg-config`` package names to link with the module.

    pkg_config_dirs
        A list of directories to add to the pkg-config search paths (extends
        the ``PKG_CONFIG_PATH`` environment variable).

    Defaults can also be specified in the ``[cython-defaults]`` section, for
    example::

        [cython-defaults]
        include_dirs = /usr/include/bar

        [cython-module: foo.one]
        sources = foo/one.pyx

        [cython-module: foo.two]
        sources = foo/two.pyx
        include_dirs = /usr/include/foo

    Here, ``foo.one`` and ``foo.two`` both will have ``/usr/include/bar`` in
    their ``include_dirs``. List parameters in defaults are extended, so in the
    example above, module ``foo.two`` ``include_dirs`` will be
    ``['/usr/include/bar', '/usr/include/foo']``.

    There are two approaches when distributing Cython modules: with or without
    the C files. Both approaches have their advantages and inconvenients:

        * not distributing the C files means they are generated on the fly when
          compiling the modules. Cython needs to be installed on the system,
          and it makes your package a bit more future proof, as Cython evolves
          to support newer Python versions. It also introduces some variance in
          the builds, as they now depend on the Cython version installed on the
          system;

        * when you distribute the C files with your package, the modules can be
          compiled directly with the host compiler, no Cython required. It also
          makes your tarball heavier, as Cython generates quite verbose code.
          It might also be good for performance-critical code, when you want to
          make sure the generated code is optimal, regardless of version of
          Cython installed on the host system.

    In the first case, you can make Cython available to pip for compilation by
    adding it to your ``setup.cfg``::

        [options]
        install_requires = cython

    This way people who just want to install your package won't need to have
    Cython installed in their system/venv.

    It is up to you to choose one option or the other. The *cythonize* argument
    controls the default mode of operation: set it to ``True`` if you don't
    distribute C files with your package (the default), and ``False`` if you
    do.

    Packages that distribute C files may use the ``CYTHONIZE`` environment
    variable to create or update the C files::

        CYTHONIZE=1 python setup.py build_ext --inplace

    You can also enable profiling for the Cython modules with the
    ``PROFILE_CYTHON`` environment variable::

        PROFILE_CYTHON=1 python setup.py build_ext --inplace

    Debugging symbols can be added with::

        DEBUG=1 python setup.py build_ext --inplace

    """
    this_dir = op.dirname(__file__)
    setup_cfg_file = op.join(this_dir, 'setup.cfg')
    cythonize = _str_to_bool(os.environ.get('CYTHONIZE', cythonize))
    profile_cython = _str_to_bool(os.environ.get('PROFILE_CYTHON', False))
    debug = _str_to_bool(os.environ.get('DEBUG', False))
    if op.exists(setup_cfg_file):
        # Create Cython Extension objects
        with open(setup_cfg_file) as fp:
            parsed_setup_cfg = parse_setup_cfg(fp, cythonize=cythonize)
        cython_ext_modules = create_cython_ext_modules(
            parsed_setup_cfg,
            profile_cython=profile_cython,
            debug=debug
        )

        if cythonize:
            try:
                from Cython.Build import cythonize
            except ImportError:
                pass
            else:
                cython_ext_modules = cythonize(cython_ext_modules)

        ext_modules = kwargs.setdefault('ext_modules', [])
        ext_modules.extend(cython_ext_modules)

    setuptools.setup(**kwargs)


def create_cython_ext_modules(cython_modules, profile_cython=False,
                              debug=False):
    """
    Create :class:`~distutils.extension.Extension` objects from
    *cython_modules*.

    *cython_modules* must be a dict, as returned by :func:`parse_setup_cfg`.

    If *profile_cython* is true, Cython modules are compiled to support Python
    proiflers.

    Debug symbols are included if *debug* is true.
    """
    if profile_cython:
        from Cython.Distutils import Extension
    else:
        from distutils.extension import Extension

    ret = []
    for name, mod_data in cython_modules.items():
        kwargs = {'name': name}
        kwargs.update(mod_data)
        if profile_cython:
            cython_directives = kwargs.setdefault('cython_directives', {})
            cython_directives['profile'] = True
        if debug:
            for args_name in ('extra_compile_args', 'extra_link_args'):
                args = kwargs.setdefault(args_name, [])
                if '-g' not in args:
                    args.append('-g')
        ext = Extension(**kwargs)
        ret.append(ext)
    return ret


def parse_setup_cfg(fp, cythonize=False, pkg_config=None, base_dir=''):
    """
    Parse the cython specific bits in a setup.cfg file.

    *fp* must be a file-like object opened for reading.

    *pkg_config* may be a callable taking a list of library names and returning
    a ``pkg-config`` like string (e.g. ``-I/foo -L/bar -lbaz``). The default is
    to use an internal function that actually runs ``pkg-config`` (normally
    used for testing).

    *base_dir* can be used to make relative paths absolute.
    """
    if pkg_config is None:
        pkg_config = _run_pkg_config
    config = configparser.SafeConfigParser()
    config.readfp(fp)
    return _expand_cython_modules(config, cythonize, pkg_config, base_dir)


class _StoreOrderedArgs(argparse.Action):

    def __call__(self, parser, namespace, values, option_string=None):
        if 'ordered_args' not in namespace:
            setattr(namespace, 'ordered_args', [])
        namespace.ordered_args.append((self.dest, values))


def extract_args(args_str, args):
    """
    Extract *args* from arguments string *args_str*.

    Return a *(extracted_args, remaining_args_str)* tuple, where
    *extracted_args* is a dict containing the extracted arguments, and
    *remaining_args_str* a string containing the remaining arguments.
    """
    parser = argparse.ArgumentParser()
    for arg in args:
        parser.add_argument(arg, action=_StoreOrderedArgs)
    args_list = shlex.split(args_str)
    try:
        args_ns, other_args = parser.parse_known_args(args_list)
    except SystemExit:
        raise Exception('args parsing failed')
    extracted_args = {}
    for arg_name, value in getattr(args_ns, 'ordered_args', []):
        arg_values = extracted_args.setdefault(arg_name, [])
        arg_values.append(value)
    return extracted_args, ' '.join(other_args)


def _expand_cython_modules(config, cythonize, pkg_config, base_dir):
    ret = {}
    for section in config.sections():
        if section.startswith(MODULE_SECTION_PREFIX):
            module_name = section[len(MODULE_SECTION_PREFIX):].strip()
            module_dict = _expand_one_cython_module(config, section, cythonize,
                                                    pkg_config, base_dir)
            ret[module_name] = module_dict
    return ret


def _expand_one_cython_module(config, section, cythonize, pkg_config,
                              base_dir):
    (pc_include_dirs,
     pc_extra_compile_args,
     pc_library_dirs,
     pc_libraries,
     pc_extra_link_args) = _expand_pkg_config_pkgs(config, section, pkg_config)

    module = {}
    module['language'] = _get_config_opt(config, section, 'language', None)
    module['extra_compile_args'] = \
        _get_config_list(config, section, 'extra_compile_args') + \
        pc_extra_compile_args
    module['extra_link_args'] = \
        _get_config_list(config, section, 'extra_link_args') + \
        pc_extra_link_args
    module['sources'] = _expand_sources(config, section, module['language'],
                                        cythonize)
    include_dirs = _get_config_list(config, section, 'include_dirs')
    include_dirs += pc_include_dirs
    include_dirs = _eval_strings(include_dirs)
    include_dirs = _make_paths_absolute(include_dirs, base_dir)
    library_dirs = _get_config_list(config, section, 'library_dirs')
    library_dirs += pc_library_dirs
    library_dirs = _eval_strings(library_dirs)
    library_dirs = _make_paths_absolute(library_dirs, base_dir)
    libraries = _get_config_list(config, section, 'libraries')
    module['include_dirs'] = include_dirs
    module['library_dirs'] = library_dirs
    module['libraries'] = libraries + pc_libraries
    all_conf_items = config.items(section)
    try:
        all_conf_items += config.items(DEFAULTS_SECTION)
    except configparser.NoSectionError:
        pass
    for key, value in all_conf_items:
        if key != 'pkg_config_packages' and key not in module:
            module[key] = value
    return module


def _make_paths_absolute(paths, base_dir):
    return [op.join(base_dir, p) if not p.startswith('/') else p
            for p in paths]


def _eval_strings(values):
    ret = []
    for value in values:
        if value.startswith('eval(') and value.endswith(')'):
            ret.append(eval(value[5:-1]))
        else:
            ret.append(value)
    return ret


def _expand_pkg_config_pkgs(config, section, pkg_config):
    pkg_names = _get_config_list(config, section, 'pkg_config_packages')
    if not pkg_names:
        return [], [], [], [], []

    original_pkg_config_path = os.environ.get('PKG_CONFIG_PATH', '')
    pkg_config_path = original_pkg_config_path.split(":")
    pkg_config_path.extend(_get_config_list(config, section,
                                            'pkg_config_dirs'))
    env = os.environ.copy()
    env['PKG_CONFIG_PATH'] = ":".join(pkg_config_path)

    extra_compile_args = pkg_config(pkg_names, '--cflags', env)
    extra_link_args = pkg_config(pkg_names, '--libs', env)

    extracted_args, extra_compile_args = extract_args(extra_compile_args,
                                                      ['-I'])
    include_dirs = extracted_args.get('I', [])
    extracted_args, extra_link_args = extract_args(extra_link_args,
                                                   ['-L', '-l'])
    library_dirs = extracted_args.get('L', [])
    libraries = extracted_args.get('l', [])

    extra_compile_args = shlex.split(extra_compile_args)
    extra_link_args = shlex.split(extra_link_args)

    return (include_dirs, extra_compile_args, library_dirs, libraries,
            extra_link_args)


def _run_pkg_config(pkg_names, command, env):
    return subprocess.check_output(['pkg-config', command] + pkg_names,
                                   env=env).decode('utf8')


def _expand_sources(config, section, language, cythonize):
    if cythonize:
        ext = CYTHON_EXT
    elif language == 'c++':
        ext = CPP_EXT
    else:
        ext = C_EXT
    sources = _get_config_list(config, section, 'sources')
    return [_replace_cython_ext(s, ext) for s in sources]


def _replace_cython_ext(filename, target_ext):
    root, ext = op.splitext(filename)
    if ext == CYTHON_EXT:
        return root + target_ext
    return filename


def _get_default(config, option, default):
    try:
        return config.get(DEFAULTS_SECTION, option)
    except (configparser.NoOptionError, configparser.NoSectionError):
        return default


def _get_config_opt(config, section, option, default):
    try:
        return config.get(section, option)
    except configparser.NoOptionError:
        return _get_default(config, option, default)


def _get_config_list(config, section, option):
    defaults_value = _get_default(config, option, '')
    try:
        value = config.get(section, option)
    except configparser.NoOptionError:
        value = ''
    return ('%s %s' % (defaults_value, value)).split()


def _str_to_bool(value):
    if isinstance(value, bool):
        return value
    value = value.lower()
    if value in ('1', 'on', 'true', 'yes'):
        return True
    elif value in ('0', 'off', 'false', 'no'):
        return False
    raise ValueError('invalid boolean string %r' % value)
